@@ -1,11 +1,11 @@
 // -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 
-#define container vector         // Could be 'vector' or 'deque' (both work but there may be performance differences)
+#define container vector      // Could be 'vector' or 'deque' (both work but there may be performance differences)
 #define USE_UNORDERED_MAP true   // set to true for unordered_map; comment out to use plain stl map.
 
 #define STRICT_R_HEADERS
 #include <Rcpp.h>
-#include <cmath>
+
 
 #include <string.h>
 #include <iostream>
@@ -15,10 +15,10 @@
 #include <utility>
 #include <iterator>
 
-using namespace std;
+
 using namespace Rcpp; 
 
-typedef container<signed int> mycont;  // a mycont  is a container [vector or deque] of *signed* ints.
+typedef std::container<signed int> mycont;  // a mycont  is a container [vector or deque] of *signed* ints.
 
 #ifdef USE_UNORDERED_MAP
 class MyVecHasher
@@ -28,7 +28,7 @@ public:
        {
               // thanks to Steffan Hooper for advice
               std::size_t seed = 0;
-              for (auto& i : vec){
+              for (const auto& i : vec){
                   seed ^= i + 0x9e3779b9 + (seed << 6) + (seed >> 2);
               }
               return seed;
@@ -44,14 +44,13 @@ typedef map<mycont, double > spray;
 
 spray prepare(const IntegerMatrix M, const NumericVector d){
     spray S;
-    mycont v;
-    signed int i,j;
     spray::iterator it;
+    mycont v;
 
-    for(i=0; i<M.nrow() ; i++){
+    for(int i=0; i<M.nrow() ; i++){
         if(d[i] != 0){
                 v.clear();
-                for(j=0; j<M.ncol(); j++){
+                for(int j=0; j<M.ncol(); j++){
                     v.push_back(M(i,j));
                 }
                 S[v] += d[i];
@@ -70,7 +69,7 @@ spray prepare(const IntegerMatrix M, const NumericVector d){
     return(S);
 }
 
-IntegerMatrix makeindex(const spray S){  // takes a spray, returns the matrix of indices
+IntegerMatrix makeindex(const spray &S){  // takes a spray, returns the matrix of indices
     const unsigned int ncol = S.begin()->first.size();
     IntegerMatrix  out(S.size(),ncol);   // index
     mycont v;
@@ -87,7 +86,7 @@ IntegerMatrix makeindex(const spray S){  // takes a spray, returns the matrix of
     return(out);
 }
 
-NumericVector makevalue(const spray S){  // takes a spray, returns data
+NumericVector makevalue(const spray &S){  // takes a spray, returns data
     NumericVector  out(S.size());   // data
     unsigned int i=0;
     spray::const_iterator it;   // it iterates through a sparse array
@@ -131,53 +130,41 @@ List spray_add
  const IntegerMatrix &M1, const NumericVector &d1,
  const IntegerMatrix &M2, const NumericVector &d2
  ){
-     spray S1, S2;   // basic idea is S1 = S1+S2;
-     spray::const_iterator it;   // it iterates through a sparse array
-     mycont v;
-     
-    S1 = prepare(M1, d1);
-    S2 = prepare(M2, d2);
+     spray S1 = prepare(M1, d1);
+     spray S2 = prepare(M2, d2);
     
-    // the "meat" of this function, namely S1=S1+S2 (S1 += S2):
-    for (it=S2.begin(); it != S2.end(); ++it){
-        v = it->first;
-        S1[v] += S2[v];   //S1 to be returned.  NB: S1 may have increased in size.
-        if(S1[v]==0){S1.erase(v);}
-    }
-
-    return retval(S1);
+     for (spray::const_iterator it=S2.begin(); it != S2.end(); ++it){
+       const mycont v = it->first;
+       S1[v] += S2[v]; // the meat:  S1=S1+S2 (S1 += S2)
+       if(S1[v]==0){S1.erase(v);}
+     }
+     
+     return retval(S1);
 }
 
 
 spray prod //
 (
- const spray S1, const spray S2
+ const spray &S1, const spray &S2
  ){
     spray Sout;
-    spray::const_iterator it1,it2;
     mycont vsum;
-    unsigned int i;
 
     // the "meat" of this function:  Sout=S1*S2
-    for (it1=S1.begin(); it1 != S1.end(); ++it1){
+    for (spray::const_iterator it1=S1.begin(); it1 != S1.end(); ++it1){
         const mycont v1 = it1->first;
         const double x1 = it1->second;
-        for (it2=S2.begin(); it2 != S2.end(); ++it2){
+        for (spray::const_iterator it2=S2.begin(); it2 != S2.end(); ++it2){
             const mycont v2 = it2->first;
             const double x2 = it2->second;
             vsum.clear();
-            for(i=0; i<v1.size(); i++){
+            for(int i=0; i<v1.size(); i++){
                 vsum.push_back(v1[i] + v2[i]);  // meat 1: powers add
             }
             Sout[vsum] += x1*x2;                // meat 2: coefficients multiply
         }
     }
-    //    for(spray::iterator it=S3.begin(); it != S3.end(); ++it){
-    //    if(it->second ==0){S3.erase(it);}
-    // }
-
     return Sout;
-
 }    
 
 spray unit //
@@ -205,16 +192,12 @@ List spray_overwrite // something like S1[ind(S2)] <- S2
  const IntegerMatrix &M1, const NumericVector &d1,
  const IntegerMatrix &M2, const NumericVector &d2
  ){
-    spray S1,S2;
-    mycont v;
-    spray::const_iterator it;
+    spray S1 = prepare(M1, d1);
+    spray S2 = prepare(M2, d2);    
     
-    S1 = prepare(M1, d1);
-    S2 = prepare(M2, d2);    
-
-    for (it=S2.begin(); it != S2.end(); ++it){
-        v = it->first;
-        S1[v] = S2[v];   // the meat
+    for (spray::const_iterator it=S2.begin(); it != S2.end(); ++it){
+      const mycont v = it->first;
+      S1[v] = S2[v];   // the meat
     }
 
     return retval(S1);
@@ -228,14 +211,14 @@ NumericVector spray_accessor // returns S1[]
  ){
     spray S;
     mycont v;
-    signed int i,j,k=0;
+    signed int k=0;
     NumericVector out(Mindex.nrow());
     
     S = prepare(M, d);
 
-    for(i=0; i<Mindex.nrow() ; i++){
+    for(int i=0; i<Mindex.nrow() ; i++){
         v.clear();
-        for(j=0; j<Mindex.ncol(); j++){
+        for(int j=0; j<Mindex.ncol(); j++){
             v.push_back(Mindex(i,j));
         }
         out[k++] = S[v];
@@ -243,23 +226,20 @@ NumericVector spray_accessor // returns S1[]
     return out;
 }
 
-
 // [[Rcpp::export]]
 List spray_setter // effectively S[M] <- d; return S
 (
  const IntegerMatrix &M1, const NumericVector &d1,
  const IntegerMatrix &M2, const NumericVector &d2    // M2 -> index ; d2 -> value
  ){
-    spray S1,S2;
     mycont v;
-    signed int i,j;
     
-    S1 = prepare(M1, d1);
-    S2 = prepare(M2, d2);
+    spray S1 = prepare(M1, d1);
+    spray S2 = prepare(M2, d2);
 
-    for(i=0; i<M2.nrow() ; i++){
+    for(int i=0; i<M2.nrow() ; i++){
         v.clear();
-        for(j=0; j<M2.ncol(); j++){
+        for(int j=0; j<M2.ncol(); j++){
             v.push_back(M2(i,j));
         }
         S1[v] = S2[v];
@@ -274,19 +254,15 @@ bool spray_equality // S1 == S2
  const IntegerMatrix &M1, const NumericVector &d1,
  const IntegerMatrix &M2, const NumericVector &d2
  ){
-    mycont v;
-    spray S1, S2;  
-    spray::const_iterator it;
-    
-    S1 = prepare(M1, d1);
-    S2 = prepare(M2, d2);
+    spray S1 = prepare(M1, d1);
+    spray S2 = prepare(M2, d2);
 
     if(S1.size() != S2.size()){
-        return FALSE;  // this line is never executed because different-sized objects are trapped by R
+        return FALSE;  // this line is never executed in package idiom, because different-sized objects are trapped by R
     }
 
-    for (it=S1.begin(); it != S1.end(); ++it){
-        v = it->first;
+    for(spray::const_iterator it=S1.begin(); it != S1.end(); ++it){
+        const mycont v = it->first;
         if(S1[v] != S2[v]){
             return FALSE;
         } else {
@@ -311,14 +287,13 @@ List spray_asum_include
  ){
     spray S;
     mycont v;
-    signed int i,j,k;
 
-    for(i=0; i<M.nrow() ; i++){
+    for(int i=0; i<M.nrow() ; i++){
         v.clear();
-        for(j=0; j<M.ncol(); j++){
+        for(int j=0; j<M.ncol(); j++){
             v.push_back(M(i,j));
         }
-        for(k=0 ;  k<n.size() ; k++){
+        for(int k=0 ;  k<n.size() ; k++){
             v[n[k]-1] = 0;    // off-by-one issue dealt with here: if n[k]=0 this means dimension 1.
         }
         S[v] += d[i];
@@ -326,7 +301,6 @@ List spray_asum_include
     return retval(S);
 }
 
-
 // [[Rcpp::export]]
 List spray_asum_exclude 
 (
@@ -335,11 +309,10 @@ List spray_asum_exclude
  ){
     spray S;
     mycont v;
-    signed int i,j;
 
-    for(i=0; i<M.nrow() ; i++){
+    for(int i=0; i<M.nrow() ; i++){
         v.clear();
-        for(j=0; j<n.size(); j++){
+        for(int j=0; j<n.size(); j++){
             v.push_back(M(i,n[j]-1));   //off-by-one error
         }
         S[v] += d[i];
@@ -357,22 +330,21 @@ List spray_deriv
 ){
     spray S;
     mycont v;
-    signed int i,j,nn;
 
     IntegerMatrix Mout(M.nrow(),M.ncol());
     NumericVector dout(d.size());
 
     // create local copies of M and d:
-    for(i=0 ; i<M.nrow() ; i++){
+    for(int i=0 ; i<M.nrow() ; i++){
         dout[i] = d[i];
-        for(j=0 ; j<M.ncol() ; j++){
+        for(int j=0 ; j<M.ncol() ; j++){
             Mout(i,j) = M(i,j);
         }
     }
     
-    for(i=0; i<Mout.nrow() ; i++){
-        for(j=0; j<Mout.ncol() ; j++){
-            nn = n[j];
+    for(int i=0; i<Mout.nrow() ; i++){
+        for(int j=0; j<Mout.ncol() ; j++){
+            int nn = n[j];
             while( (nn>0) & (d[i]!=0)  ){  // while loop because it might not run at all
                 dout[i] *= Mout(i,j);  // multiply d first, then decrement M (!)
                 Mout(i,j)--;
@@ -380,7 +352,7 @@ List spray_deriv
             }
         }
         v.clear();
-        for(j=0; j<Mout.ncol() ; j++){
+        for(int j=0; j<Mout.ncol() ; j++){
             v.push_back(Mout(i,j));
         }                    
         S[v] += dout[i];   // increment because v is not row-unique any more
@@ -394,19 +366,16 @@ List spray_pmax
  const IntegerMatrix &M1, const NumericVector &d1,
  const IntegerMatrix &M2, const NumericVector &d2 
  ){
-    spray S1,S2;
-    spray::const_iterator it;   // it iterates through a sparse array
-    
-    S1 = prepare(M1, d1);
-    S2 = prepare(M2, d2);
+    spray S1 = prepare(M1, d1);
+    spray S2 = prepare(M2, d2);
 
-    for (it=S1.begin(); it != S1.end(); ++it){
+    for (spray::const_iterator it = S1.begin(); it != S1.end(); ++it){
         const mycont v = it->first;
         if(S2[v] > S1[v]){ S1[v] = S2[v];} // S1[v] = max(S1[v],S2[v]);
         S2.erase(v); // not S2[v] = 0;  // OK because the iterator is it1 and this line modifies S2
     }
             
-    for (it=S2.begin(); it != S2.end(); ++it){ //iterate through S2 keys not in S1
+    for (spray::const_iterator it = S2.begin(); it != S2.end(); ++it){ //iterate through S2 keys not in S1
         const mycont v = it->first;
         if(S2[v] > 0){ S1[v] = S2[v]; }
     }
@@ -420,19 +389,16 @@ List spray_pmin
  const IntegerMatrix &M1, const NumericVector &d1,
  const IntegerMatrix &M2, const NumericVector &d2 
  ){
-    spray S1,S2;
-    spray::const_iterator it;   // it iterates through a sparse array
-    
-    S1 = prepare(M1, d1);
-    S2 = prepare(M2, d2);
+    spray S1 = prepare(M1, d1);
+    spray S2 = prepare(M2, d2);
 
-    for (it=S1.begin(); it != S1.end(); ++it){
+    for (spray::const_iterator it = S1.begin(); it != S1.end(); ++it){
         const mycont v = it->first;
         if(S2[v] < S1[v]){ S1[v] = S2[v]; }// S1[v] = min(S1[v],S2[v]);
         S2.erase(v);
     }
             
-    for (it=S2.begin(); it != S2.end(); ++it){
+    for (spray::const_iterator it = S2.begin(); it != S2.end(); ++it){
         const mycont v = it->first;
         if(S2[v] < 0){S1[v] = S2[v]; } // S1[v] = min(S2[v],0);
     }
@@ -461,6 +427,3 @@ List spray_power
     }
     return retval(out);
 }
-
-
-
